# evaluate projections with different bootstraps
# save evaluated projections in files so that we can later use them to evaluate accuracy of the classifier
# also evaluate projections on all examples so that we can later compute the error on learn data
import orngRegistry
orngRegistry.addWidgetDirectories()

import orange, time, os, sys
from orngVizRank import *
import cPickle, time
import orngVisFuncts

projectionDir = os.path.join(os.getcwd(), "bootstrap")
if not os.path.exists(projectionDir):
    os.mkdir(projectionDir)

names = ["leukemia.tab", "DLBCL.tab", "prostata.tab", "MLL.tab", "SRBCT.tab", "lung.tab"]

projCount = 50000
attrCount = 8
attrSelMeasure = GAMMA_ALL

vizrank = VizRank(RADVIZ)
vizrank.attrCont = CONT_MEAS_S2NMIX
vizrank.optimizationType = EXACT_NUMBER_OF_ATTRS
vizrank.projectionLimit = projCount
vizrank.optimizeProjectionLimit = 0
vizrank.attributeCount = attrCount
vizrank.attrSubsetSelection = attrSelMeasure
        
permutationIndices = {}
B = 100
datasetIndices = {}
random.seed(0)
bootstrapFile = os.path.join(projectionDir, "bootstrapIndices.txt")
if os.path.exists(bootstrapFile):
    f = open(bootstrapFile, "r")
    datasetIndices = cPickle.load(f)
    f.close()
else:
    for filename in names:
        examples = orange.ExampleTable(filename, dontCheckStored = 1)
        n = len(examples)
        allIndices = range(n)*B
        random.shuffle(allIndices)
        datasetIndices[filename] = allIndices
    f = open(bootstrapFile, "w")
    cPickle.dump(datasetIndices, f)
    f.close()

start = 0
#if len(sys.argv) > 1:
#    start = int(sys.argv[1])

for b in range(start, B):
    for filename in names:
        examples = orange.ExampleTable(filename, dontCheckStored = 1)
        n = len(examples)
        allIndices = datasetIndices[filename]
        selIndices = allIndices[b*n:(b+1)*n]

        learnset = examples.getitemsref(selIndices)
        testset = examples.getitemsref([i for i in range(n) if i not in selIndices])
        if not learnset or not testset:  continue
        
        datasetDir = os.path.join(projectionDir, filename[:-4]) # make folders for saving projections
        if not os.path.exists(datasetDir):
            os.mkdir(datasetDir)
            
        print b, time.ctime(), filename

        projFName = "B%02d-%s.proj" % (b, filename[:-4])
        fullProjFName = os.path.join(datasetDir, projFName)
        if os.path.exists(fullProjFName): continue

        vizrank.setData(learnset)
        vizrank.evaluateProjections()
        vizrank.save(fullProjFName, count = 100)    # save computed projections


for filename in names:
    examples = orange.ExampleTable(filename, dontCheckStored = 1)

    datasetDir = os.path.join(projectionDir, filename[:-4]) # make folders for saving projections
    if not os.path.exists(datasetDir):
        os.mkdir(datasetDir)
        
    print time.ctime(), filename

    fullProjFName = os.path.join(datasetDir, "All-%s.proj" % (filename[:-4]))
    if os.path.exists(fullProjFName): continue

    vizrank.setData(examples)
    vizrank.evaluateProjections()
    vizrank.save(fullProjFName, count = 100)    # save computed projections

